<?php
/*
Plugin Name: Acts as Group
Plugin URI: http://ideasilo.wordpress.com/2009/07/28/acts-as-group/
Description: Acts as Group provides a means for communication on multi-user blogs.
Author: Takayuki Miyoshi
Author URI: http://ideasilo.wordpress.com/
Version: 1.1
*/

/*  Copyright 2009 Takayuki Miyoshi (email: takayukister at gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

define( 'AAG_VERSION', '1.1' );

if ( ! defined( 'AAG_PLUGIN_BASENAME' ) )
	define( 'AAG_PLUGIN_BASENAME', plugin_basename( __FILE__ ) );

if ( ! defined( 'AAG_PLUGIN_NAME' ) )
	define( 'AAG_PLUGIN_NAME', trim( dirname( AAG_PLUGIN_BASENAME ), '/' ) );

if ( ! defined( 'AAG_PLUGIN_DIR' ) )
	define( 'AAG_PLUGIN_DIR', WP_PLUGIN_DIR . '/' . AAG_PLUGIN_NAME );

if ( ! defined( 'AAG_ADD_GROUP_CAPABILITY' ) )
	define( 'AAG_ADD_GROUP_CAPABILITY', 'read' );

if ( ! defined( 'AAG_JOIN_GROUP_CAPABILITY' ) )
	define( 'AAG_JOIN_GROUP_CAPABILITY', 'read' );

function aag_plugin_path( $path = '' ) {
	return path_join( AAG_PLUGIN_DIR, trim( $path, '/' ) );
}

function aag_plugin_url( $path = '' ) {
	global $wp_version;

	if ( version_compare( $wp_version, '2.8', '<' ) ) { // Using WordPress 2.7
		$path = path_join( AAG_PLUGIN_NAME, $path );
		return plugins_url( $path );
	}

	return plugins_url( $path, __FILE__ );
}

function aag_admin_url( $file, $query = array() ) {
	$file = trim( $file, ' /' );
	if ( 'admin/' != substr( $file, 0, 6 ) )
		$file = 'admin/' . $file;

	$path = 'admin.php';
	$path .= '?page=' . AAG_PLUGIN_NAME . '/' . $file;

	if ( $query = build_query( $query ) )
		$path .= '&' . $query;

	$url = admin_url( $path );

	return $url;
}


// Pre-2.8 compatibility
if ( ! function_exists( 'esc_js' ) ) {
	function esc_js( $text ) {
		return js_escape( $text );
	}
}

if ( ! function_exists( 'esc_html' ) ) {
	function esc_html( $text ) {
		return wp_specialchars( $text );
	}
}

if ( ! function_exists( 'esc_attr' ) ) {
	function esc_attr( $text ) {
		return attribute_escape( $text );
	}
}

if ( ! function_exists( 'esc_sql' ) ) {
	function esc_sql( $text ) {
		global $wpdb;
		return $wpdb->escape( $text );
	}
}


function aag_db_table( $table_name ) {
	global $wpdb;

	if ( isset( $wpdb->base_prefix ) ) { // WPMU
		$prefix = $wpdb->base_prefix;
	} else {
		$prefix = $wpdb->prefix;
	}

	return $prefix . trim( $table_name );
}

require_once AAG_PLUGIN_DIR . '/includes/functions.php';
require_once AAG_PLUGIN_DIR . '/includes/capabilities.php';
require_once AAG_PLUGIN_DIR . '/includes/install.php';

if ( is_admin() )
	require_once AAG_PLUGIN_DIR . '/admin/admin.php';

if ( version_compare( $wp_version, '2.8', '>=' ) ) // Using WordPress 2.8+
	require_once AAG_PLUGIN_DIR . '/includes/widget.php';

function aag_load_plugin_textdomain() {
	load_plugin_textdomain( 'aag',
		'wp-content/plugins/acts-as-group/languages', 'acts-as-group/languages' );
}

add_action( 'init', 'aag_load_plugin_textdomain' );


/* Filters */

function aag_make_atmark_clickable( $text ) {
	if ( empty( $text ) )
		return $text;

	return preg_replace_callback( '/(?<=^|\s)@([a-zA-Z0-9._-]+)(?=$|\s)/',
		'aag_make_atmark_clickable_cb', $text );
}

function aag_make_atmark_clickable_cb( $matches ) {
	$user_login = $matches[1];

	if ( ! $user = get_userdatabylogin( $user_login ) )
		return $matches[0];

	$link_url = aag_admin_url( 'messages.php', array( 'author_id' => $user->ID ) );
	return '@<a href="' . $link_url . '">' . $matches[1] . '</a>';
}

add_filter( 'aag_message_content', 'wptexturize' );
add_filter( 'aag_message_content', 'make_clickable' );
add_filter( 'aag_message_content', 'links_add_target' );
add_filter( 'aag_message_content', 'convert_smilies' );
add_filter( 'aag_message_content', 'aag_make_atmark_clickable' );

?>