<?php

class AagWidget extends WP_Widget {

	function AagWidget() {
		$widget_ops = array(
			'description' => __( "The most recent messages flow on your groups", 'aag' ) );
		parent::WP_Widget(false, __( 'Group Messages', 'aag' ), $widget_ops );
	}

	function widget( $args, $instance ) {
		extract( $args );

		if ( $group_id = (int) $instance['group'] )
			$group = aag_get_group( $group_id );

		if ( ! $title = trim( $instance['title'] ) ) {
			if ( $group )
				$title = stripslashes( $group->group_name );
			else
				$title = __( 'Group Messages', 'aag' );
		}

		if ( ! $number = (int) $instance['number'] )
			$number = 10;
		else if ( $number < 1 )
			$number = 1;
		else if ( $number > 15 )
			$number = 15;

		echo $before_widget;
		echo $before_title . esc_html( $title ) . $after_title;

		// Only logged-in users can see the messages.
		if ( is_user_logged_in() ) {

			if ( $group_id && ! aag_you_can_message_group( $group_id ) ) {
				echo '<p>'
					. esc_html( __( "You don't have a membership for this group.", 'aag' ) )
					. '</p>';
				echo '<ul>';
				$join_url = aag_admin_url( 'join-group.php' );
				echo '<li><a href="' . $join_url . '">' . esc_html( __( 'Join now') ) . '</a></li>';
				echo '</ul>';

			} else {
				$messages = aag_your_group_timeline(
					array( 'perpage' => $number, 'group_id' => $group_id ) );
				echo '<ul>';
				foreach ( $messages as $message ) {
					$class = 'group-' . $message->group_id;
					$class .= ' author-' . $message->user_id;

					echo '<li class="' . $class . '">';

					$user = get_userdata( $message->user_id );
					echo '<strong><a href="' . aag_admin_url( 'messages.php', array( 'author_id' => $message->user_id ) ) . '" title="' . esc_attr( stripslashes( $user->display_name ) ) . '">' . esc_html( $user->user_login ) . '</a></strong> ';

					echo apply_filters( 'aag_message_content',
						stripslashes( $message->message_content ) );

					echo '</li>';
				}
				echo '</ul>';

				$more_url = aag_admin_url( 'messages.php', array( 'group_id' => $group_id ) );
				echo '<ul><li><em><a href="' . $more_url . '">'
					. esc_html( __( 'more', 'aag' ) )
					. '</a></em></li></ul>';
			}
		} else {
			echo '<ul><li>';
			wp_loginout();
			echo '</li></ul>';
		}

		echo $after_widget;
	}

	function update( $new_instance, $old_instance ) {
		$instance = $old_instance;
		$instance['title'] = strip_tags( $new_instance['title'] );
		$instance['number'] = (int) $new_instance['number'];
		$instance['group'] = (int) $new_instance['group'];
		return $instance;
	}

	function form( $instance ) {
		$title = esc_attr( $instance['title'] );
		if ( ! $number = (int) $instance['number'] ) {
			$number = 5;
		}
		$group_id = (int) $instance['group'];

	?>
		<p><label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e( 'Title:', 'aag' ); ?></label>
		<input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" type="text" value="<?php echo $title; ?>" /></p>

		<p><label for="<?php echo $this->get_field_id( 'group' ); ?>"><?php _e( 'Group to show:', 'aag' ); ?></label>
		<select id="<?php echo $this->get_field_id( 'group' ); ?>" name="<?php echo $this->get_field_name( 'group' ); ?>">
	<?php
		echo '<option value=""' . ( ( ! $group_id ) ? ' selected="selected"' : '' ) . '>'
			. __( '--- Show All ---', 'aag' ) . '</option>';

		$groups = aag_get_groups( array( 'perpage' => 100 ) );
		foreach ( $groups as $group ) {
			$selected = ( $group->group_id == $group_id ) ? ' selected="selected"' : '';
			echo '<option value="' . $group->group_id . '"' . $selected . '>'
				. esc_html( stripslashes( $group->group_name ) ) . '</option>';
		}
	?>
		</select></p>

		<p><label for="<?php echo $this->get_field_id( 'number' ); ?>"><?php _e( 'Number of messages to show:', 'aag' ); ?></label>
		<input id="<?php echo $this->get_field_id( 'number' ); ?>" name="<?php echo $this->get_field_name( 'number' ); ?>" type="text" value="<?php echo $number; ?>" size="3" /><br />
		<small><?php _e( '(at most 15)', 'aag' ); ?></small></p>
	<?php
	}

}

function aag_widgets_init() {
	register_widget( 'AagWidget' );
}

add_action( 'widgets_init', 'aag_widgets_init' );

?>