<?php
/*
Plugin Name: Fitr Theme Options
Plugin URI: http://wpwave.com/plugins/fitr-theme-options/
Description: Helps theme designers to make a rich option page for themes. See readme.txt or plugin URL to learn how can you add FTO features to your theme. It's really easy.
Author: Hassan Jahangiry
Version: 0.82
Author URI: http://wpwave.com/

	This plugin is released under GPL:
	http://www.opensource.org/licenses/gpl-license.php
*/


define('FTO_VERSION','0.82');
define('FTO_PATH',WP_PLUGIN_URL.'/'.basename(dirname(__FILE__)));
if (__FILE__ == $_SERVER['SCRIPT_FILENAME'] ) die();
load_plugin_textdomain('fto', false, FTO_PATH . '/lang');


if( 'WordPress Default'==get_option('current_theme') ) {
	//Register Fitr Theme Options for default theme. Designers can copy and use these lines to register Fitr Theme Options for their own theme.
	if ( function_exists('register_fitr_theme_options') ) {
		register_fitr_theme_options(array(
					'blogtitle'=>'#headerimg h1',
					'blogdescription'=>'#headerimg .description',
					'header_background'=>'#headerimg', //Maybe for background color, background image or logo. It's require to enter width and height for image and logo.
					'headerimg_path'=>'images/kubrickheader.jpg', //Default header image e.g. 'images/header.jpg'
					'headerimg_width'=>'760', //pixel
					'headerimg_height'=>'200', 
					'container'=>'#page', //  Main blog div e.g. .wrap .maincol or #page
					'feedicon'=>'#feed',	  
					'menubar'=>'',	//Menu container e.g. #menu or #nav
					'menuli_bg'=>'',	//e.g. #nav ul li span
					'menuli_bghover'=>'',	//You can use this color for submenus or current page. separate with comma(,) e.g. #nav li:hover, .current_page
					'menuli_a'=>'',	//Add a 
					'postbox'=>'.post',  // Post container includes title, content, etc
					'posttitle'=>'.post h2,.post h3',  
					'postcontent'=>'.entry',
					'postmeta'=>'.narrowcolumn .postmetadata',
					'date'=>'.post small',
					'author'=>'',
					'trackback'=>'',
					'allowed_html'=>'',
					'widgettitle'=>'#sidebar ul li h2',
					'sidebarbody'=>'#sidebar',
					'widgetbox'=>'#sidebar ul li',
					'widgetli_a'=>'#sidebar ul li ul li a', //Add a
					'footer'=>'#footer'
					
					)
		);
	}
}

if( 'WordPress Classic'==get_option('current_theme') ) {
	//Register Fitr Theme Options for default theme. Designers can copy and use these lines to register Fitr Theme Options for their own theme.
	if ( function_exists('register_fitr_theme_options') ) {
		register_fitr_theme_options(array(
					'blogtitle'=>'#header',
					'blogdescription'=>'',
					'header_background'=>'#header', //Maybe for background color, background image or logo. It's require to enter width and height for image and logo.
					'headerimg_path'=>'', //Default header image e.g. 'images/header.jpg'
					'headerimg_width'=>'', //pixel
					'headerimg_height'=>'', 
					'container'=>'#rap', //  Main blog div e.g. .wrap .maincol or #page
					'feedicon'=>'',	  
					'menubar'=>'',	//Menu container e.g. #menu or #nav
					'menuli_bg'=>'',	//e.g. #nav ul li span
					'menuli_bghover'=>'',	//You can use this color for submenus or current page. separate with comma(,) e.g. #nav li:hover, .current_page
					'menuli_a'=>'',	//Add a 
					'postbox'=>'.post',  // Post container includes title, content, etc
					'posttitle'=>'.storytitle,.post h3',  
					'postcontent'=>'.storycontent',
					'postmeta'=>'.meta',
					'date'=>'#content h2',
					'author'=>'',
					'trackback'=>'',
					'allowed_html'=>'',
					'sidebarbody'=>'#menu ',
					'widgettitle'=>'#menu ul li',
					'widgetbox'=>'#menu ul li',
					'widgetli_a'=>'#menu ul li ul li a', //Add a
					'footer'=>'.credit'
					
					)
		);
	}
}







/**
 * Theme designers use this function to register Fitr Theme Options for their theme.
 * @param string|array $args List of CSS elements see above lines or plugin URL for full document.
 * @return Always true.
 */
function register_fitr_theme_options( $args = array()) {
	global $fitr_registered_options;
	
	if ( is_string( $args) )
		parse_str( $args, $args);
		
	$fitr_registered_options = (array) $args;
	
	return true;
}



/**
 * Theme designers use this function to register different styles for their theme. Selected stylesheet will be import to header.
 * @uses $fitr_registered_styles Global variable
 * @param string $name name of stylesheet. 
 * @param string $path Relative path to stylesheet e.g. 'styles/red.css' template_directory will be added to the first of path.
 * @param string $screenshot Relative path to screenshot image 100x100 e.g. 'styles/red-screenshot.jpg' template_directory will be added to the first of path.
 * @return Always true.
 */
function fto_register_style( $name, $path, $screenshot ) {
	global $fitr_registered_styles;
	$fitr_registered_styles[] = array( 'name'=>$name, 'path'=>$path, 'screenshot'=>$screenshot );
	return true;
}



/**
 * Theme designers use this function to add custom fields to Fitr Theme Option page. The field value will be saved in fto_settings option. For example if we added a field with name 'welcome_message' we should use this code to get value: $settings=get_option('fto_settings'); $welcome=$settings['welcome_message'];
 * @uses $fto_custom_fields Global variable
 * @param string $name name of fields. 
 * @param text|checkbox|number|color|textarea|dropdown|cat|page $type Type of field.
 * @param string $desc Description for user. 
 * @param string $options (optional) Different options for  dropdown and radio types. separated with '|'.   
 * @return always true.
 */
function fto_add_custom_field( $name, $type, $desc, $options='' ) {
	global $fto_custom_fields;
	$options = explode( '|',trim( $options, " \n\t\r\0\x0B\|") ) ;
	$fto_custom_fields[] = array( 'name'=>$name, 'type'=>$type, 'description'=>$desc, 'options'=> $options );
	return true;
}



/**
 * Includes FTO Option Page.
 */
function fitr_theme_options_page() {
	include('fto_admin_page.php');
}



/**
 * Main plugin function. This function uses to convert plugin settings to compressed, easy to read CSS code. 
 * @param: An array of admin page $_POST values. 
 */
function fto_make_style( $post = array() ){
	global $fitr_registered_options;
	
	if ( ( $post['fto_header_hide'] ) || ( $post['fto_headerbg_color'] )  )   
		$output.=$fitr_registered_options['header_background'].'{background-image:none;background-color:#'.$post['fto_headerbg_color'].";}\n";
	
	if ( ( $fitr_registered_options['blogtitle'] ) && ( ( $post['fto_blogtitle_color'] ) || ( $post['fto_blogtitle_size'] ) || ( $post['fto_blogtitle_font'] ) || ( $post['fto_blogtitle_align'] ) || ( $post['fto_blogtitle_hide'] ) ) ) : 
		
		if ( $post['fto_blogtitle_hide'] )  {
			$hide_items[]=$fitr_registered_options['blogtitle']; 
		}else{
			if ( strpos( $fitr_registered_options['blogtitle'],',') ) { 
					$btitle_items=explode(',',$fitr_registered_options['blogtitle'] );
					
					foreach ( $btitle_items as $btitle_item) {
						$output.=$btitle_item.','.$btitle_item.' a,'.$btitle_item.' a:visited,'.$btitle_item.' a:hover'.','; 
					}
						
					$output=substr( $output, 0, strlen( $output)-1 ); //remove last (,)	
					$output.='{';//start
			}else{
				$output.=$fitr_registered_options['blogtitle'].','.$fitr_registered_options['blogtitle'].' a,'.$fitr_registered_options['blogtitle'].' a:visited,'.$fitr_registered_options['blogtitle'].' a:hover'.'{'; //start
			}
			
			if ( $post['fto_blogtitle_color'] ) 
				$output.='color: #'.$post['fto_blogtitle_color'].';';
				
			if ( $post['fto_blogtitle_size'] ) 
				$output.='font-size: '.$post['fto_blogtitle_size'].'px;';
				
			if ( $post['fto_blogtitle_font'] )  {
				$output.='font-family: ';
				switch( $post['fto_blogtitle_font'] ) {
					case '1':
						$output.='Arial, Helvetica, sans-serif;';
						break;
					case '2':
						$output.='Times New Roman, Times, serif;';
						break;
					case '3':
						$output.='Courier New, Courier, monospace;';
						break;
					case '4':
						$output.='Georgia, Times New Roman, serif;';
						break;
					case '5':
						$output.='Verdana, Arial, Helvetica, sans-serif;';
						break;
					case '6':
						$output.='Geneva, Arial, Helvetica, sans-serif;';
						break;
					case '7':
						$output.='Tahoma, sans-serif;';
						break;
				}
			}
				
			
			switch( $post['fto_blogtitle_align'] ) { // add some padding for better appearance
				case 'left' :
					$output.='padding-left:20px;text-align: left;';
					break;
				case 'right' :
					$output.='padding-right:20px;text-align: right;';
					break;
				case 'center' :
					$output.='text-align: center;';
					break;
			}
			
			$output.="}\n"; //end
		
		}
	endif; 
	
	
	if ( ( $fitr_registered_options['blogdescription'] ) && ( ( $post['fto_blogdescription_color'] ) || ( $post['fto_blogdescription_size'] ) || ( $post['fto_blogdescription_font'] ) || ( $post['fto_blogdescription_align'] ) || ( $post['fto_blogdescription_hide'] ) ) ) : 
		
		if ( $post['fto_blogdescription_hide'] )  {
			$hide_items[]=$fitr_registered_options['blogdescription']; 
		}else{
			if ( strpos( $fitr_registered_options['blogdescription'],',') ) { 
					$bdesc_items=explode( ',',$fitr_registered_options['blogdescription'] );
					
					foreach ( $bdesc_items as $bdesc_item) {
						$output.=$bdesc_item.','.$bdesc_item.' a,'.$bdesc_item.' a:visited,'.$bdesc_item.' a:hover'.','; 
					}
						
					$output=substr( $output,0,strlen( $output)-1); //remove last (,)	
					$output.='{';//start
			}else{
				$output.=$fitr_registered_options['blogdescription'].','.$fitr_registered_options['blogdescription'].' a,'.$fitr_registered_options['blogdescription'].' a:visited,'.$fitr_registered_options['blogdescription'].' a:hover'.'{'; //start
			}
			
			if ( $post['fto_blogdescription_color'] ) 
				$output.='color: #'.$post['fto_blogdescription_color'].';';
				
			if ( $post['fto_blogdescription_size'] ) 
				$output.='font-size: '.$post['fto_blogdescription_size'].'px;';
				
			if ( $post['fto_blogdescription_font'] )  {
				$output.='font-family: ';
				switch( $post['fto_blogdescription_font'] ) {
					case '1':
						$output.='Arial, Helvetica, sans-serif;';
						break;
					case '2':
						$output.='Times New Roman, Times, serif;';
						break;
					case '3':
						$output.='Courier New, Courier, monospace;';
						break;
					case '4':
						$output.='Georgia, Times New Roman, serif;';
						break;
					case '5':
						$output.='Verdana, Arial, Helvetica, sans-serif;';
						break;
					case '6':
						$output.='Geneva, Arial, Helvetica, sans-serif;';
						break;
					case '7':
						$output.='Tahoma, sans-serif;';
						break;
				}
			}
				
			switch( $post['fto_blogdescription_align'] ) { // add some padding for better appearance
				case 'left' :
					$output.='padding-left:20px;text-align: left;';
					break;
				case 'right' :
					$output.='padding-right:20px;text-align: right;';
					break;
				case 'center' :
					$output.='text-align: center;';
					break;
			}
			
			$output.="}\n"; //end
		
		}
	endif; 
	
	
	if ( ( $post['fto_body_bgcolor'] ) || ( $post['fto_bodybgimg_url'] ) || ( $post['fto_bodybg_format'] ) )  : 
		
			$output.='body{'; //start
			
			if ( $post['fto_body_bgcolor'] ) 
				$output.='background-image:none;background-color: #'.$post['fto_body_bgcolor'].';';
				
			if ( $post['fto_bodybgimg_url'] ) 
				$output.='background-image:url(\''.$post['fto_bodybgimg_url'].'\');';
				
			if ( $post['fto_bodybg_format'] ) {
				$output.='background-repeat:';
				
				switch( $post['fto_bodybg_format'] ) {
					case 'no-repeat-fixed':
						$output.='no-repeat;background-attachment:fixed;';
						break;
					case 'x-repeat-fixed':
						$output.='repeat-x;background-attachment:fixed;';
						break;
					case 'x-repeat':
						$output.='repeat-x;';
						break;
					case 'y-repeat':
						$output.='repeat-y;';
						break;
					case 'y-repeat':
						$output.='repeat;';
						break;
				}
			}
			
			$output.="}\n"; //end
	endif;
	
	
	if ( ( $post['fto_link_color'] ) || ( $post['fto_link_hover'] ) || ( $post['fto_link_format'] ) )  : 
		
			$output.='a,a:visited{'; //start
			
			if ( $post['fto_link_color'] ) 
				$output.='color: #'.$post['fto_link_color'].';';
				
			if ( $post['fto_link_format'] ) {
				if ('underdash'!=$post['fto_link_format'] )
					$output.='text-decoration:'.$post['fto_link_format'].';';
				else
					$output.='text-decoration:none;border-bottom:1px dashed #ccc;';
			}
			$output.="}\n";	 //end
				
			if ( $post['fto_link_hover'] ) 
				$output.='a:hover {color:#'.$post['fto_link_hover']."}\n";
			
	endif;
	
	if ( ( $fitr_registered_options['container'] ) && ( $post['fto_container_bgcolor'] ) ) ://start
		$output.=$fitr_registered_options['container'].'{background-image:none;background-color:#'.$post['fto_container_bgcolor'].';';
		$output.="background-image:none;}\n";//end
	endif;
	
	if ( (( 'rtl'==$post['fto_text_direction'] ) && (! file_exists(STYLESHEETPATH.'/rtl.css') )) ||  (('ltr'==$post['fto_text_direction']) && (!file_exists(STYLESHEETPATH.'/ltr.css'))) ) :
	
		if ( ( $fitr_registered_options['container'] ) || ( $fitr_registered_options['postcontent'] ) ) {
			$direction_items=array('container','blogtitle','blogdescription','postbox','posttitle','postcontent','postmeta','sidebarbody','widgetbox','widgettitle','footer');
			
			foreach ( $direction_items as $dir_item ) {
				if ( $fitr_registered_options[$dir_item] )
					$output.=$fitr_registered_options[$dir_item].',';
			}
				
			$output=substr( $output,0,strlen( $output)-1 ); //remove last (,)
			$output.='{direction:'.$post['fto_text_direction'].";}\n";
			
			
			$align_items=array('posttitle','postcontent','postmeta','sidebarbody','widgetbox','widgettitle');
			foreach ( $align_items as $align_item ) {
				if ( $fitr_registered_options[$align_item] )
					$output.=$fitr_registered_options[$align_item].',';
			}
					
					
			$output=substr($output,0,strlen( $output)-1); //remove last (,)
			
			if ( 'rtl'==$post['fto_text_direction'] )
				$output.="{text-align:right;}\n";
			else
				$output.="{text-align:left;}\n";
				
			
				
		}
	endif;
	
	if ( $post['fto_feedicon_hide'] )  
			$hide_items[]=$fitr_registered_options['feedicon']; 
		
	
	if ( $post['fto_image_styles'] ) :
		$output.="p img {padding: 0;max-width: 100%;}img.centered {display: block;margin-left: auto;margin-right: auto;}img.alignright {padding: 4px;margin: 0 0 2px 7px;display: inline;}img.alignleft {padding: 4px;margin: 0 7px 2px 0;display: inline;}.alignright {float: right;}.alignleft {float: left;}.aligncenter,div.aligncenter {display: block;margin-left: auto;margin-right: auto;}a img {border: none;}.wp-caption {border: 1px solid #ddd;text-align: center;background-color: #f3f3f3;padding-top: 4px;margin: 10px;}.wp-caption img {margin: 0;padding: 0;border: 0 none;}.wp-caption p.wp-caption-text {font-size: 11px;line-height: 17px;padding: 0 4px 5px;margin: 0;
}\n";
	endif;
	
	if ( $post['fto_menubar_hide'] )  {
			$hide_items[]=$fitr_registered_options['menubar'];
			$hide_items[]=$fitr_registered_options['menuli_bg'];
			$hide_items[]=$fitr_registered_options['menuli_a'];	
	} 
	
	
	if ( ( $fitr_registered_options['menubar'] )&&( $post['fto_menubar_bgcolor'] )&& (!$post['fto_menubar_hide'] ) )
		$output.=$fitr_registered_options['menubar'].'{background-image:none;background-color:#'.$post['fto_menubar_bgcolor']."}\n";
		
	if ( ( $fitr_registered_options['menuli_bg'] )&&( $post['fto_menu_bgcolor'] )&& (!$post['fto_menubar_hide'] ) )
		$output.=$fitr_registered_options['menuli_bg'].'{background-image:none;background-color:#'.$post['fto_menu_bgcolor']."}\n";
		
	if ( ( $fitr_registered_options['menuli_bghover'] )&&( $post['fto_menu_bghover'] )&& (!$post['fto_menubar_hide'] ) )
		$output.=$fitr_registered_options['menuli_bghover'].'{background-image:none;background-color:#'.$post['fto_menu_bghover']."}\n";
		
	if ( ( $fitr_registered_options['menuli_a'] )&&( $post['fto_menuli_a_color'] )&& (!$post['fto_menubar_hide'] ) ) {
		$output.=$fitr_registered_options['menuli_a'].',';
		
			if ( strpos( $fitr_registered_options['menuli_a'],',') )  { //Add visited to different elements
				$output.=$str_replace(',',':visited,',$fitr_registered_options['menuli_a'] );
				$output.=':visited{';
			}else{
				$output.=$fitr_registered_options['menuli_a'].':visited{';
			}
			
			$output.='color:#'.$post['fto_menuli_a_color']."}\n";
		
	}
	
		
	if ( ( $fitr_registered_options['menuli_a'] )&&( $post['fto_menuli_ahover_color'] )&& (!$post['fto_menubar_hide'] ) ) {
			if ( strpos( $fitr_registered_options['menuli_a'],',') )  {//Add hover to different elements
				$output.=$str_replace(',',':hover,',$fitr_registered_options['menuli_a'] ).':hover{';
			}else{
				$output.=$fitr_registered_options['menuli_a'].':hover{';
			}
	
			$output.='color:#'.$post['fto_menuli_ahover_color']."}\n";
	}
		
	
	if ( ( $fitr_registered_options['menuli_a'] )&&( $post['fto_menuli_a_size'] )&& (!$post['fto_menubar_hide'] ))
		$output.=$fitr_registered_options['menuli_a'].'{font-size:'.$post['fto_menuli_a_size']."px;}\n";
	
	
	if ( ( $fitr_registered_options['postbox'] ) && (( $post['fto_postbox_bgcolor'] ) || ( $post['fto_postbox_border'] ))) :
		$output.=$fitr_registered_options['postbox'].'{';
		
		if ( $post['fto_postbox_bgcolor'] ) 
			$output.='background-image:none;padding:10px;background-color: #'.$post['fto_postbox_bgcolor'].';';
			
		if ( $post['fto_postbox_border'] ) {
			switch( $post['fto_postbox_border'] ) {
				case 'none':
						$output.='border:none;';
						break;
				case 'fullborder':
					$output.='border:1px solid #ccc;';
					break;
				case 'dashedborder':
					$output.='border:1px dashed #ccc;';
					break;
				case 'topborder':
					$output.='border:none;border-top:1px solid #ccc;';
					break;
				case 'botborder':
					$output.='border:none;border-bottom:1px solid #ccc;';
					break;
				case 'topbotborder':
					$output.='border:none;border-top:1px solid #ccc;border-bottom:1px solid #ccc;';
					break;
			}
		}
		$output.="}\n";
			
	endif;
	
	
	if ( ( $fitr_registered_options['posttitle'] ) && ( ( $post['fto_posttitle_color'] ) || ( $post['fto_posttitle_size'] ) || ( $post['fto_posttitle_font'] ) || ( $post['fto_posttitle_border'] )  ) ) : 
		
			//Separate different elements for a.a:visited and a:hover
			if (strpos( $fitr_registered_options['posttitle'],',')) { 
				$ptitle_items=explode(',',$fitr_registered_options['posttitle'] );
				
				foreach ( $ptitle_items as $ptitle_item) 
					$output.=$ptitle_item.','.$ptitle_item.' a,'.$ptitle_item.' a:visited,'.$ptitle_item.' a:hover'.','; //start
					
				$output=substr( $output,0,strlen( $output)-1); //remove last (,)	
				$output.='{';//start
			}else{
				$output.=$fitr_registered_options['posttitle'].','.$fitr_registered_options['posttitle'].' a,'.$fitr_registered_options['posttitle'].' a:visited,'.$fitr_registered_options['posttitle'].' a:hover'.'{'; //start
			}
			
			
			
			if ( $post['fto_posttitle_color'] ) 
				$output.='color: #'.$post['fto_posttitle_color'].';';
				
			if ( $post['fto_posttitle_size'] ) 
				$output.='font-size: '.$post['fto_posttitle_size'].'px;';
				
			if ( $post['fto_posttitle_font'] )  {
				$output.='font-family: ';
				switch( $post['fto_posttitle_font'] ) {
					case '1':
						$output.='Arial, Helvetica, sans-serif;';
						break;
					case '2':
						$output.='Times New Roman, Times, serif;';
						break;
					case '3':
						$output.='Courier New, Courier, monospace;';
						break;
					case '4':
						$output.='Georgia, Times New Roman, serif;';
						break;
					case '5':
						$output.='Verdana, Arial, Helvetica, sans-serif;';
						break;
					case '6':
						$output.='Geneva, Arial, Helvetica, sans-serif;';
						break;
					case '7':
						$output.='Tahoma, sans-serif;';
						break;
				}
			}

			$output.="}\n";
			
			//Fix a multi-border problem
			if ( $post['fto_posttitle_border'] ) {
				$output.=$fitr_registered_options['posttitle'].'{';
				switch( $post['fto_posttitle_border'] ) {
					case 'none':
						$output.='border:none;';
						break;
					case 'fullborder':
						$output.='border:1px solid #ccc;padding:3px;';
						break;
					case 'dashedborder':
						$output.='border:1px dashed #ccc;padding:3px;';
						break;
					case 'topborder':
						$output.='border:none;border-top:1px solid #ccc;padding-top:2px;';
						break;
					case 'botborder':
						$output.='border:none;border-bottom:1px solid #ccc;padding-bottom:2px;';
						break;
					case 'topbotborder':
						$output.='border:none;border-top:1px solid #ccc;border-bottom:1px solid #ccc;padding-top:2px;padding-bottom:2px;';
						break;
				}
				$output.="}\n";
			}
			
			
	endif; //end post title
	
	
	if ( ( $fitr_registered_options['postcontent'] ) && ( ( $post['fto_postcontent_size'] ) || ( $post['fto_postcontent_font'] ) || ( $post['fto_postcontent_justify'] ) || ( $post['fto_postcontent_color'] )  ) ) : 
		
		
			$output.=$fitr_registered_options['postcontent'].','.$fitr_registered_options['postcontent'].' p{'; //start
			
			if ( $post['fto_postcontent_color'] ) 
				$output.='color: #'.$post['fto_postcontent_color'].';';
				
			if ( $post['fto_postcontent_size'] ) 
				$output.='font-size: '.$post['fto_postcontent_size'].'px;';
				
			if ( $post['fto_postcontent_font'] )  {
				$output.='font-family: ';
				switch( $post['fto_postcontent_font'] ) {
					case '1':
						$output.='Arial, Helvetica, sans-serif;';
						break;
					case '2':
						$output.='Times New Roman, Times, serif;';
						break;
					case '3':
						$output.='Courier New, Courier, monospace;';
						break;
					case '4':
						$output.='Georgia, Times New Roman, serif;';
						break;
					case '5':
						$output.='Verdana, Arial, Helvetica, sans-serif;';
						break;
					case '6':
						$output.='Geneva, Arial, Helvetica, sans-serif;';
						break;
					case '7':
						$output.='Tahoma, sans-serif;';
						break;
				}
			}
				
			
			if ( $post['fto_postcontent_justify'] ) 
						$output.='text-align:justify;';
						
			$output.="}\n";	//end
	endif; 
	
	
	
	if ( ( $fitr_registered_options['postmeta'] ) && ( ( $post['fto_postmeta_bgcolor'] ) || ( $post['fto_postmeta_color'] ) || ( $post['fto_postmeta_align'] ) || ( $post['fto_postmeta_hide'] )  ) ) : 
		
		if ( $post['fto_postmeta_hide'] ) {
			$hide_items[]=$fitr_registered_options['postmeta'];	
		}else{
				
			$output.=$fitr_registered_options['postmeta'].'{'; //start
			
			if ( $post['fto_postmeta_bgcolor'] ) 
				$output.='background-image:none;background-color: #'.$post['fto_postmeta_bgcolor'].';';
				
			if ( $post['fto_postmeta_color'] ) 
				$output.='color: #'.$post['fto_postmeta_color'].';';
				
			switch( $post['fto_postmeta_align'] ) { // add some padding for better appearance
				case 'left' :
					$output.='padding-left:8px;text-align: left;';
					break;
				case 'right' :
					$output.='padding-right:8px;text-align: right;';
					break;
				case 'center' :
					$output.='text-align: center;';
					break;
			}	
			$output.="}\n";	//end
		}
		
	endif; 
	
	
	if ( ( $fitr_registered_options['date'] ) &&  ( $post['fto_date_hide'] )) 
		$hide_items[]=$fitr_registered_options['date'];	
		
	if ( ( $fitr_registered_options['author'] ) && ( $post['fto_author_hide'] )) 
		$hide_items[]=$fitr_registered_options['author'];	
	
	if ( ( $fitr_registered_options['trackback'] ) && ( $post['fto_trackback_hide'] )) 
		$hide_items[]=$fitr_registered_options['trackback'];
		
	if ( ( $fitr_registered_options['allowed_html'] ) && ( $post['fto_allowed_html_hide'] )) 
		$hide_items[]=$fitr_registered_options['allowed_html'];	
	
	
	if ( ( $fitr_registered_options['sidebarbody'] ) && (( $post['fto_sidebarbody_bgcolor'] ) || ( $post['fto_sidebarbody_border'] ))) :
		$output.=$fitr_registered_options['sidebarbody'].'{';
		
		if ( $post['fto_sidebarbody_bgcolor'] ) 
			$output.='background-image:none;background-color:#'.$post['fto_sidebarbody_bgcolor'].';';
			
		if ( $post['fto_sidebarbody_border'] ) {
			switch( $post['fto_sidebarbody_border'] ) {
				case 'none':
						$output.='border:none;';
						break;
				case 'fullborder':
					$output.='border:1px solid #ccc;';
					break;
				case 'dashedborder':
					$output.='border:1px dashed #ccc;';
					break;
				case 'rightborder':
					$output.='border:none;border-right:1px solid #ccc;';
					break;
				case 'leftborder':
					$output.='border:none;border-left:1px solid #ccc;';
					break;
				case 'riglefborder':
					$output.='border:none;border-left:1px solid #ccc;border-right:1px solid #ccc;';
					break;
			}
		}
		$output.="}\n";
		
		
			
	endif;
	
	
	if ( ( $fitr_registered_options['widgetbox'] ) && ( ( $post['fto_widget_bgcolor'] ) || ( $post['fto_widget_color'] ) || ( $post['fto_wdiget_font'] )  ) ) : 
		
		
			$output.=$fitr_registered_options['widgetbox'].'{'; //start
			
			if ( $post['fto_widget_bgcolor'] ) 
				$output.='background-image:none;background-color: #'.$post['fto_widget_bgcolor'].';';
				
			if ( $post['fto_widget_color'] ) 
				$output.='color: #'.$post['fto_widget_color'].';';
				
			if ( $post['fto_wdiget_font'] )  {
				$output.='font-family: ';
				switch( $post['fto_wdiget_font'] ) {
					case '1':
						$output.='Arial, Helvetica, sans-serif;';
						break;
					case '2':
						$output.='Times New Roman, Times, serif;';
						break;
					case '3':
						$output.='Courier New, Courier, monospace;';
						break;
					case '4':
						$output.='Georgia, Times New Roman, serif;';
						break;
					case '5':
						$output.='Verdana, Arial, Helvetica, sans-serif;';
						break;
					case '6':
						$output.='Geneva, Arial, Helvetica, sans-serif;';
						break;
					case '7':
						$output.='Tahoma, sans-serif;';
						break;
				}
			}
				
			if ( $post['fto_widget_border'] ) {
				switch( $post['fto_widget_border'] ) {
					case 'none':
						$output.='border:none;';
						break;
					case 'fullborder':
						$output.='border:1px solid #ccc;';
						break;
					case 'dashedborder':
						$output.='border:1px dashed #ccc;';
						break;
					case 'rightborder':
						$output.='border:none;border-right:1px solid #ccc;';
						break;
					case 'leftborder':
						$output.='border:none;border-left:1px solid #ccc;';
						break;
					case 'riglefborder':
						$output.='border:none;border-left:1px solid #ccc;border-right:1px solid #ccc;';
						break;
				}
			}
						
		$output.="}\n";	//end
	endif; 
	
	
	if ( ( $fitr_registered_options['widgetli_a'] ) && (( $post['fto_widgetli_a_color'] ) || ( $post['fto_widgetlihover_color'] ))) :
	
	
		if ( $post['fto_widgetli_a_color'] ) {
			if (strpos( $fitr_registered_options['widgetli_a'],',')) { //Separate different elements
				$wli_items=explode(',',$fitr_registered_options['widgetli_a'] );
				
				foreach ( $wli_items as $wli_item) {
					$output.=$wli_item.','.$wli_item.':visited'.'{';
				}
				
				$output=substr( $output,0,strlen( $output)-1); //Remove last (,)	
				$output.='{';//start
			}else{
				$output.=$fitr_registered_options['widgetli_a'].','.$fitr_registered_options['widgetli_a'].':visited'.'{';
			}
			$output.='color:#'.$post['fto_widgetli_a_color'].";}\n"; 
		}
		
		if ( $post['fto_widgetlihover_color'] )  {
			if (strpos( $fitr_registered_options['widgetli_a'],',')) { //Separate different elements
				$wli_items=explode(',',$fitr_registered_options['widgetli_a'] );
				
				foreach ( $wlihover_items as $wlihover_item) {
					$output.=$wlihover_item.':hover,';
				}
				
				$output=substr( $output,0,strlen( $output)-1); //remove last (,)	
				$output.='{';//start
			}else{
				$output.=$fitr_registered_options['widgetli_a'].':hover'.'{';
			}
			$output.='color:#'.$post['fto_widgetlihover_color'].";}\n"; 
		}
		
	endif;
	
	//fix a border bug in Kubric (testing for other themes)
	if ( ( $fitr_registered_options['widgetli_a'] ) && (( $post['fto_sidebarbody_border'] )||( $post['fto_widget_border'] ) ))  {
		$widgetli=str_replace(' a','',$fitr_registered_options['widgetli_a'] ); //remove "a" (link)
		$output.=$widgetli." {border:none;}\n";
	}
	
	
	if ( ( $fitr_registered_options['widgettitle'] ) && ( ( $post['fto_widgettitle_bgcolor'] ) || ( $post['fto_widgettitle_color'] ) || ( $post['fto_widgettitle_size'] ) || ( $post['fto_widgettitle_border'] ) || ( $post['fto_widgettitle_font'] ) ) ) : 
		
			//Separate different elements for a.a:visited and a:hover
			if (strpos( $fitr_registered_options['widgettitle'],',')) { 
				$wtitle_items=explode(',',$fitr_registered_options['widgettitle'] );
				
				foreach ( $wtitle_items as $wtitle_item) {
					$output.=$wtitle_item.','.$wtitle_item.' a,'.$wtitle_item.' a:visited,'.$wtitle_item.' a:hover'.','; 
				}
				
				$output=substr( $output,0,strlen( $output)-1); //remove last (,)	
				$output.='{';//start
			}else{
				$output.=$fitr_registered_options['widgettitle'].','.$fitr_registered_options['widgettitle'].' a,'.$fitr_registered_options['widgettitle'].' a:visited,'.$fitr_registered_options['widgettitle'].' a:hover'.'{'; //start
			}
			
			if ( $post['fto_widgettitle_bgcolor'] ) 
				$output.='background-image:none;background-color: #'.$post['fto_widgettitle_bgcolor'].';';
			
			if ( $post['fto_widgettitle_color'] ) 
				$output.='color: #'.$post['fto_widgettitle_color'].';';
				
			if ( $post['fto_widgettitle_size'] ) 
				$output.='font-size: '.$post['fto_widgettitle_size'].'px;';
				 
			if ( $post['fto_widgettitle_font'] )  {
				$output.='font-family: ';
				switch( $post['fto_widgettitle_font'] ) {
					case '1':
						$output.='Arial, Helvetica, sans-serif;';
						break;
					case '2':
						$output.='Times New Roman, Times, serif;';
						break;
					case '3':
						$output.='Courier New, Courier, monospace;';
						break;
					case '4':
						$output.='Georgia, Times New Roman, serif;';
						break;
					case '5':
						$output.='Verdana, Arial, Helvetica, sans-serif;';
						break;
					case '6':
						$output.='Geneva, Arial, Helvetica, sans-serif;';
						break;
					case '7':
						$output.='Tahoma, sans-serif;';
						break;
				}
			}
			$output.="}\n";
			
			//fix a little bug in borders
			if ( $post['fto_widgettitle_border'] ) {
				$output.=$fitr_registered_options['widgettitle'].'{';
				switch( $post['fto_widgettitle_border'] ) {
					case 'none':
						$output.='border:none;';
						break;
					case 'fullborder':
						$output.='border:1px solid #ccc;padding:3px;';
						break;
					case 'dashedborder':
						$output.='border:1px dashed #ccc;padding:3px;';
						break;
					case 'topborder':
						$output.='border:none;border-top:1px solid #ccc;padding-top:2px;';
						break;
					case 'botborder':
						$output.='border:none;border-bottom:1px solid #ccc;padding-bottom:2px;';
						break;
					case 'topbotborder':
						$output.='border:none;border-top:1px solid #ccc;border-bottom:1px solid #ccc;padding-top:2px;padding-bottom:2px;';
						break;
				}
				$output.="}\n";
			}
			
			
		
		
	endif; 
	
	if ( ( $fitr_registered_options['footer'] ) &&  ( $post['fto_footer_bgcolor'] )  ) 
		$output.=$fitr_registered_options['footer'].'{background-image:none;background-color:#'.$post['fto_footer_bgcolor'].";}\n";
	
	//Now hide items
	if ( $hide_items) :
	
		foreach ( $hide_items as $hide_item) {
			$output.=$hide_item.',';
		}
			
		$output=substr( $output,0,strlen( $output)-1); //remove last (,)
		$output.="{display:none;}\n";
		
	endif;
		
	
	
	
	return $output;
}


/**
 * Includes required scripts to FTO options page header.
 */
function fto_scripts_include(){
	wp_enqueue_script( 'admin_page', FTO_PATH.'/script/admin_page.js', array( 'jquery' ));
	wp_enqueue_script( 'jscolor', FTO_PATH.'/script/jscolor.js');
	add_thickbox();
	wp_print_styles();
}



/**
 * Adds FTO options page uses fto_scripts_include to include scripts.
 */
function fitr_theme_options_init_page() {
	$page=add_submenu_page('themes.php', esc_html__('Fitr Theme Options','fto'), esc_html__('Fitr Theme Options','fto'), 'manage_options', 'fitr-options', 'fitr_theme_options_page');
	add_action("admin_print_scripts-$page",'fto_scripts_include');
}
add_action('admin_menu', 'fitr_theme_options_init_page');



/**
 * Ajax function uses for live preview 
 */
function fto_ajax_refresh(){
		check_ajax_referer( "fitr-options-ajax" );

		if ( is_string( $_POST['data'] ) ) 
			parse_str( $_POST['data'],$arr);
		
		$post = (array) $arr;
		
		foreach ( $post as $key => $value)  {
			if (!is_array( $value))
				$post[$key]=esc_attr(stripslashes( $value));
			else
				foreach ( $value as $val) {
					$post[$key][]=esc_attr(stripslashes( $val));
				}
		}
		
		$style=fto_make_style( $post);
		update_option('fto_preview',$style);
}
add_action('wp_ajax_fto_ajax_refresh', 'fto_ajax_refresh' );



/**
 * Defines custom header image. add ability to use Kubrick image maker with custom header image and redirect for upload message
 */
function fto_init() {
global $fitr_registered_options;

	//Theme does not support custome header. let's define it
	if ( (!defined('HEADER_IMAGE_WIDTH')&&( $fitr_registered_options['headerimg_width'] ) && ( $fitr_registered_options['headerimg_height'] ) && ( $fitr_registered_options['header_background'] ) ) ) {
				  
			define('HEADER_IMAGE', '%s/'.$fitr_registered_options['headerimg_path'] ); //Defulat preview image for custom header page. Maybe empty. %s is theme uri. 
			define('HEADER_IMAGE_WIDTH', $fitr_registered_options['headerimg_width'] );
			define('HEADER_IMAGE_HEIGHT', $fitr_registered_options['headerimg_height'] );
			define( 'NO_HEADER_TEXT', true );
			
			// We have not enough information to display header text correctly. Also user can hide or change blogtitle/blogdescription from FTO page so we hide header text and related buttons in custom header page
			function fto_admin_custom_header_style() { 
				?>
				<style type="text/css">
				#headimg {
					background: url(<?php header_image() ?>) no-repeat;
					height: <?php echo HEADER_IMAGE_HEIGHT; ?>px;
					width: <?php echo HEADER_IMAGE_WIDTH; ?>px;	
				}
				
				#headimg h1,#headimg #desc,#headimg h1 a,#hidetext,#pickcolor,#defaultcolor{
					display:none; 
				} 
				</style>  
				<?php		
			}
			
			function fto_custom_header_style() {
				global $fitr_registered_options;
				?>
				<style type="text/css">
				  #header {
				  background: url(<?php header_image() ?>) no-repeat;
				  }
				</style>
				<?php
			}
			
			if ( function_exists('add_custom_image_header') ) 
				add_custom_image_header('fto_custom_header_style', 'fto_admin_custom_header_style');
				
	}
	
	
	//Allow user to generate colorful headers via header-img.php for Default Theme
	if ( (function_exists('kubrick_header_image')&&(isset( $_POST['submitform'] )) && (isset( $_GET['page'] ))&&('functions.php'==$_GET['page'] ) )) 
		if(get_option('kubrick_header_image'))
			set_theme_mod('header_image',get_template_directory_uri().'/images/'.get_option('kubrick_header_image'));
		elseif (strpos(get_theme_mod('header_image'), 'header-img.php?') !== false)  
			set_theme_mod('header_image',get_template_directory_uri().'/images/kubrickheader.jpg');
			
			

	//After image cropping we redirect user to FTO page.
	if ( (isset( $_GET['page'] )) && ('custom-header'==$_GET['page'] ) && (isset( $_GET['step'] )) && ('3'==$_GET['step'] ) ) {
		$redirect_to = get_option('siteurl').'/wp-admin/admin.php?page=fitr-options&msg=upload'; //Custom Header Updated Return to fto option page
		$redirect_to = wp_sanitize_redirect( $redirect_to);
		wp_redirect( $redirect_to);
	}
}
add_action('init','fto_init');


/**
 * Adds customized style to theme header
 */
function fto_add_head() {
	echo "\n<!-- Start Fitr Theme Options Code -->\n";
	$settings=get_option('fto_settings'); 
	
	//If there is a RTL or LTR stylesheet and user want to change direction let's do that
	if ( ( ('rtl' == $settings['fto_text_direction'] )||(('rtl' == $text_direction)&& (!'ltr' == $settings['fto_text_direction'])) ) && (file_exists(STYLESHEETPATH.'/rtl.css') ) ) { ?>
		<link rel="stylesheet" href="<?php bloginfo('stylesheet_directory');?>/rtl.css" type="text/css" media="screen" />
	<?php
	}elseif ( ('ltr' == $settings['fto_text_direction'] ) && (file_exists(STYLESHEETPATH.'/ltr.css') ) ) { ?>
		<link rel="stylesheet" href="<?php bloginfo('stylesheet_directory');?>/ltr.css" type="text/css" media="screen" />
    <?php 
	}
	
	if ( ($settings['fto_import_style'] ) && (!isset( $_GET['import_style'])) ) { ?>
            <link rel="stylesheet" href="<?php echo get_template_directory_uri().'/'.$settings['fto_import_style']; ?>" type="text/css" media="screen" /><?php
    }
		
	if ( is_preview() ) { 
	
		if (!current_user_can('manage_options'))
			die( 'Sorry, you do not have the right to access this page.' );
			
	 	if ( (isset($_GET['import_style'])) && (!validate_file($_GET['import_style'])) )
            echo '<link rel="stylesheet" href="'.get_template_directory_uri().'/'.$_GET["import_style"].'" type="text/css" media="screen" />';
			
		//Display original theme (without style) for WP Theme Preview and Style Preview	
        if (isset($_GET['fto_preview'])) {
		
			if ($settings['fto_import_style']) { ?>
				<link rel="stylesheet" href="<?php echo get_template_directory_uri().'/'.$settings['fto_import_style']; ?>" type="text/css" media="screen" /><?php
			}?>
		
			<style type="text/css" media="screen">
                <?php echo get_option('fto_preview'); ?>
            </style><?php
		}
		
	}else{
		if ($settings['fto_import_style']) { ?>
            <link rel="stylesheet" href="<?php echo get_template_directory_uri().'/'.$settings['fto_import_style']; ?>" type="text/css" media="screen" /><?php
    	}
		?>
		<style type="text/css" media="screen">
			<?php echo get_option('fto_style'); ?>
		</style>
		<?php
	}
    echo "\n<!-- End Fitr Theme Options Code -->\n";
}
add_action('wp_head', 'fto_add_head', 99);


/**
 * Add user footer code to theme footer
 */
function fto_add_footer() {
	$settings=get_option('fto_settings');
	echo $settings['fto_footer_code'];
}
add_action('wp_footer', 'fto_add_footer');



/**
 * Delete FTO settings and uses theme default
 */
function fto_restore() {
	delete_option('fto_style'); 
	delete_option('fto_settings');
	delete_option('fto_preview');
	delete_option('kubrick_header_image');
	delete_option('kubrick_header_color');
	delete_option('kubrick_header_display');
	remove_theme_mods(); //Restore header image
}
add_action('switch_theme','fto_restore'); //Remove old settings and prepare for new theme



/**
 * If site is based on WordPress MU returns true else returns false
 */
function fto_is_wpmu() { 
	return function_exists(switch_to_blog);
}


/**
 * Activation hook
 */
function fto_activate() {
	add_option('fto_settings','','This will store Fitr Theme Options settings');
	add_option('fto_style','','This will store CSS code generated by Fitr Theme Options');
	add_option('fto_preview','','This will store preview CSS code generated by Fitr Theme Options');
}
register_activation_hook(__FILE__, 'fto_activate');


/**
 * Dectivation hook (Unistall)
 */
function fto_deactivate() {
	fto_restore();
}
register_deactivation_hook(__FILE__, 'fto_deactivate');

?>