<?php
/*
Plugin Name: Greg's High Performance SEO
Plugin URI: http://counsellingresource.com/features/2009/07/23/high-performance-seo/
Description: Configure over 100 separate on-page SEO characteristics. Load just 600 lines of code per page view. No junk: just high performance SEO at its best.
Version: 1.0
Author: Greg Mulhauser
Author URI: http://counsellingresource.com/
*/

/*  Copyright 2009 Greg Mulhauser

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

class gregsHighPerformanceSEO {

var $plugin_prefix; // prefix for option names and post meta tables

function gregsHighPerformanceSEO($plugin_prefix='') {
$this->__construct($plugin_prefix);
return;
} 

function __construct($plugin_prefix='') {
$this->plugin_prefix = $plugin_prefix . '_';
// set up to enable dupe content handling, head description and keywords, and robots
add_filter('the_content',array(&$this,'paged_comments_dupefix'),$this->opt('comment_page_replacement_level'));
add_action('wp_head', array(&$this,'head_desc'), 2);
add_action('wp_head', array(&$this,'head_keywords'), 3);
if ($this->opt('index_enable')) {
   remove_action('wp_head', 'noindex', 1);
   add_action('wp_head', array(&$this,'robots'), 4);
   } // end index modifications
return;
} // end constructor

function opt($name) { // get our options
return get_option($this->plugin_prefix . $name);
} // end option retriever

function opt_clean($name) { // get cleaned up options
return stripslashes(wp_specialchars_decode($this->opt($name),ENT_QUOTES));
} // end clean option retriever

function get_meta($post_id='', $key='', $single=false) { // get post meta
return get_post_meta($post_id,'_' . $this->plugin_prefix . $key,$single);
} // end replacement for get_post_meta

function get_meta_clean($post_id='', $key='', $single=false) { // get cleaned up post meta
return stripslashes(wp_specialchars_decode($this->get_meta($post_id,$key,$single),ENT_QUOTES));
} // end clean meta retriever

function prepout($content) { // general cleanup of text in preparation for display
return wptexturize(convert_chars(force_balance_tags($content)));
} // end cleaner upper

function titlecase($string, $forcelower = false) { // conversion to title case
// note this doesn't work for words in quotes, because ucfirst doesn't work
if (!$this->opt('title_case')) return $string;
$exceptions = str_replace(",", " ",$this->opt('title_case_exceptions'));
$exceptions = str_replace("  ", "",$exceptions);
$exceptions = explode(" ", $exceptions);
$words = explode(" ", $string);
$newwords = array();
foreach ($words as $word) {
   if (!in_array($word, $exceptions)) {
	  if (strtoupper($word) != $word) { // mess with it only if not already all caps
		  $word = ($forcelower) ? ucfirst(strtolower($word)) : ucfirst($word);
		  }
	   }
   array_push($newwords, $word);
}
return ucfirst(join(" ", $newwords)); // ucfirst again in case first word was in the exception list
} // end titlecase

function get_comment_page() { // check for whether we're on a paged comment page
global $wp_query,$post,$overridden_cpage;
// the following line checks for $overridden_cpage getting set by WP function 'comments_template', which may happen in case some other rude plugin is output buffering the entire page, corrupting the value of the query var cpage that we need; note it will only be set if the original cpage was empty
if ( $overridden_cpage ) return false;
$page = get_query_var('cpage');
if ( !$page )
	$page = 0;
if ( $page < 1 )
	return false;
else return true;
} // end check for comment page

function get_type_key() { // what kind of page are we on?
global $wp_query,$paged,$post;
$key = '';
if (is_front_page() && (!is_paged())) $key = 'frontnotpaged';
elseif (is_front_page() && is_paged()) $key = 'frontispaged';
elseif (is_single()) $key = 'single';
elseif (is_tag()) $key = 'tag';
elseif (is_author()) $key = 'author';
elseif (is_search()) $key = 'search';
elseif (is_category()) $key = 'category';
elseif (is_page()) $key = 'page';
elseif (is_date()) {
	if (is_year()) $key = 'year';
	elseif (is_month()) $key = 'month';
	elseif (is_day()) $key = 'day';
	else $key = 'otherdate';
	} // end handling date-based archives
elseif (is_404()) $key = '404';
return $key;
} // end setting type of page

function get_category_quick ($post) { // grab cat(s) for this post
$cats = get_the_category ($post->ID);
if (count ($cats) > 0) {
  foreach ($cats AS $cat)
		$category[] = $cat->cat_name;
  $category = implode (', ', $category);
return $category;
} else return '';
}

function strip_para($content,$leavebreaks='false') {
// Drop paragraph tags wrapped around content
$stripped = preg_replace('/<p.*?>([\w\W]+?)<\/p>/','$1 ',$content); // put in extra space in case of multiple paragraphs
if (!$leavebreaks) $stripped = str_replace(array('<br />',"\n","\r"),' ',$stripped); // drop breaks too
$stripped = str_replace('  ',' ',$stripped); // kill double spaces introduced while dropping breaks
return trim($stripped);
} // end stripping paragraph tags

function get_swaps($type='') { // return replacements necessary to construct titles and descriptions
// the returned array holds all our option base names for main (_title) and secondary (_title_secondary) titles and secondary descriptions (_desc), plus arrays with any additional swapping that needs to be done in addition to the basics already included locally by whatever function is calling this one
global $wp_query,$paged,$post;
$secondary = (($type == 'single') || ($type == 'page')) ? $this->get_secondary_title() : '';
$full_url = ($type == '404') ? 'http://' . str_replace('\\','/',htmlspecialchars(strip_tags(stripslashes($_SERVER['SERVER_NAME']))) .   htmlspecialchars(strip_tags(stripslashes($_SERVER['REQUEST_URI'])))) : '';
$cat_desc = ($type == 'category') ? wp_specialchars_decode($this->strip_para(stripslashes(category_description()),$this->opt('cat_desc_leave_breaks')),ENT_QUOTES) : '';
$cat_of_post = ($type == 'single') ? $this->titlecase($this->get_category_quick($post)) : '';
$tag_desc = (($type == 'tag') && function_exists('tag_description')) ? $this->strip_para(tag_description(),$this->opt('tag_desc_leave_breaks')) : '';
return array (
	"frontnotpaged" => array ('home',''),
	"frontispaged" => array ('home_paged',''),
	"single" => array ('post',array("%post_title%" => single_post_title('',false), "%post_title_custom%" => $secondary, "%category_title%" => $cat_of_post)),
	"tag" => array ('tag',array("%tag_title%" => $this->titlecase(single_tag_title('',false)),"%tag_desc%" => $tag_desc)),
	"author" => array ('author',array("%author_name%" => $this->get_author())),
	"search" => array ('search',array("%search_terms%" => strip_tags(stripslashes(get_search_query())))),
	"category" => array ('category',array("%category_title%" => $this->titlecase(single_cat_title('',false)),"%category_desc%"=>$cat_desc)),
	"page" => array ('page',array("%page_title%" => ltrim(wp_title('',false)), "%page_title_custom%" => $secondary)),
	"year" => array ('year_archive',array("%year%" => get_the_time('Y'))),
	"month" => array ('month_archive',array("%month%" => get_the_time('F, Y'))),
	"day" => array ('day_archive',array("%day%" => get_the_time('F jS, Y'))),
	"otherdate" => array ('other_date_archive',''),
	"paged" => array ('paged_modification',array("%page_number%" => $paged, "%page_total%" => intval($wp_query->max_num_pages))),
	"404" => array('404',array("%error_url%" => $full_url)),
);
} // end setting array of swaps

function select_desc_comments() { // select and construct the secondary description to use for comment pages
global $post;
if ($this->get_comment_page()) {
   $desc = $this->opt_clean('comment_desc_replacement');
   ($this->opt('comment_desc_replacement_override') && $this->opt('enable_secondary_titles')) ?
			  $title_for_insertion = $this->get_secondary_title() :
			  $title_for_insertion = single_post_title('',false);
   $desc = str_replace('%post_title%',$title_for_insertion,$desc);
   $desc = str_replace('%comment_page%',get_query_var('cpage'),$desc);
   } // end handling comment pages
   else $desc = get_the_excerpt();
return $desc;
} // end getting secondary description for comments pages

function select_desc($echo=true) { // select and construct the secondary description if enabled
global $post;
$desc = '';
$suffix = '_desc';
if ($this->opt('enable_secondary_desc')) {
	$default = $this->opt_clean('secondary_desc_override_text');
	if ($this->opt('secondary_desc_override_all') && !$this->get_comment_page()) {
		$desc = $default;
		if (($desc == '') && !($this->opt('secondary_desc_use_blank')))
			$desc = get_the_excerpt();
		}
	elseif (is_single() || is_page()) {
		if ($this->get_comment_page() && $this->opt('paged_comments_descfix'))
		   $desc = $this->select_desc_comments();
		else {
		   $desc = $this->get_meta_clean($post->ID, 'secondary_desc', true);
		   if (($desc == '') && has_excerpt())
			   $desc = get_the_excerpt();
		   if (($desc == '') || ($this->opt('secondary_desc_override_excerpt')))
			   $desc = $default;
		   if (($desc == '') && !($this->opt('secondary_desc_use_blank')))
			   $desc = get_the_excerpt();
			 } // end handling single posts and pages not comments
		} // end handling single posts and pages
	else {
		 $key = $this->get_type_key();
		 $swap = array(
					  "%blog_name%" => get_bloginfo('name'),
					  "%blog_desc%" => get_bloginfo('description'),
					  );
		 $descswaps = $this->get_swaps($key);
		 if ($key != '') {
			 $desc = $this->opt_clean($descswaps[$key]['0'] . $suffix);
			 // Special handling for tag archives, so we can use a tag description if one is specified under 2.8+, or fall back to a different description if not
			 if (($key == 'tag') && ($descswaps[$key]['1']['%tag_desc%'] != '') && $this->opt('tag_desc_override'))
				  $desc = $this->opt_clean($descswaps[$key]['0'] . $suffix . '_extra');
			 // end special handling for tag archives
			 if ($desc == '') $desc = $default; // if blank, use default
			 if (is_array($descswaps[$key]['1'])) $swap = array_merge($swap,$descswaps[$key]['1']);
			 }	
			 else $desc = $default;
		 $desc = str_replace(array_keys($swap), array_values($swap),$desc);
		 } // end handling other than single posts and pages and overrides
	} else { $desc = ''; }// end handling with secondary desc enabled
$desc = $this->prepout($desc);
if ($this->opt('secondary_desc_wrap')) $desc = wpautop($desc); // wrap only if requested
if ($echo) echo $desc;
	else return $desc;
return;
} // end getting secondary description

function get_legacy_title() { // grab titles stored by old SEO plugins
global $post;
$legacy = '';
if ($this->opt('enable_secondary_titles_legacy')) {
	$legacy = get_post_meta($post->ID, 'title', true);
	if ('' == $legacy) $legacy = get_post_meta($post->ID, '_headspace_page_title', true);
	if (('' == $legacy) && $this->opt('enable_seott')) { // SEO Title Tag is slightly more involved
	   $seott = $this->opt_clean('seott_key_name');
	   if ('' != $seott) $legacy = get_post_meta($post->ID, $seott, true);
	   } // end handling SEO Title Tag data
	} // end handling legacy titles
return $legacy;
} // end getting legacy titles

function get_secondary_title() { // select the secondary title, if enabled
global $post;
if ($this->opt('enable_secondary_titles')) {
	$secondary = $this->get_meta_clean($post->ID, 'secondary_title', true);
	if ('' != $secondary) return $this->prepout($secondary);
	elseif ($this->opt('enable_secondary_titles_legacy') && !$this->opt('legacy_title_invert')) $secondary = $this->get_legacy_title();
	if ('' != $secondary) return $this->prepout(stripslashes(wp_specialchars_decode($secondary, ENT_QUOTES)));
	} // end of secondary titles enabled
$secondary = ltrim(wp_title('',false));
return $secondary;
} // end getting secondary title

function get_comment_page_title($ismain = false) { // construct the title for paged comment pages
global $post;
$title_for_insertion = '';
if ($this->get_comment_page() && $this->opt('paged_comments_titlefix')) {
   $title = stripslashes($this->opt('comment_title_replacement'));
   if ($this->opt('comment_title_replacement_override') && !$ismain && $this->opt('enable_secondary_titles')) // do not override if main title
		$title_for_insertion = $this->get_secondary_title();
   else {
		if ($this->opt('legacy_title_invert') && $this->opt('enable_secondary_titles_legacy')) 
			$title_for_insertion = $this->get_legacy_title();
		if ('' == $title_for_insertion) $title_for_insertion = single_post_title('',false);
   } // end check in case of legacy title inversion
   $title = str_replace('%post_title%',$title_for_insertion,$title);
   $title = str_replace('%comment_page%',get_query_var('cpage'),$title);
   } // end handling comment pages
   else $title = ltrim(wp_title('',false));
return $title; // output will still need texturizing, but that's OK
} // end getting comment page title

function select_title($main=true,$echo=true) { // root function for titles calls on other functions to produce the title, depending on type of page; $main controls whether to return main or secondary title
global $post;
if (is_single() || is_page()) 
   $title = ($this->get_comment_page()) ? $this->get_comment_page_title($main) : $this->get_other_titles($main);
   // end handling singles and pages
else $title = (($this->opt('main_for_secondary')) || $main) ? $this->get_other_titles(true) : $this->get_other_titles(false);
   // end handling pages other than singles and pages
$title = $this->prepout(stripslashes(wp_specialchars_decode($title, ENT_QUOTES)));
if ($echo) echo $title;
	else return $title;
return;
} // end select title

function get_other_titles($main=false) { // get titles for other than paged comments; $main controls whether to return main or secondary title
global $wp_query,$paged,$post;

$suffix = ($main || !($this->opt('enable_secondary_titles'))) ? '_title' : '_title_secondary';

$swap = array("%blog_name%" => get_bloginfo('name'));

$key = $this->get_type_key();

$titleswaps = $this->get_swaps($key);

if ($key != '') {
	$title = $this->opt_clean($titleswaps[$key]['0'] . $suffix);
	if (is_array($titleswaps[$key]['1'])) $swap = array_merge($swap,$titleswaps[$key]['1']);
	}	
	else $title = wp_title('| ',false,'right') . get_bloginfo('name'); // if it was none of these, just get the usual

if ((($key == 'single') || ($key == 'page')) && ($main && $this->opt('legacy_title_invert') && $this->opt('enable_secondary_titles_legacy'))) { // handle legacy titles as main titles
	$title = $this->get_legacy_title();
	} // end handling screwy legacy titles as main titles

if ($title == '') $title = ltrim(wp_title('',false));
if (is_paged()) { // modify with something like a page number, if this is paged?
	$modifier = $this->opt_clean($titleswaps['paged']['0'] . $suffix); // do some trickery to modify the title for paging
	if ($modifier != '') $title = str_replace('%prior_title%',$title,$modifier); 
	$swap = array_merge($swap,$titleswaps['paged']['1']);
	} // end handling paged
$title = str_replace(array_keys($swap), array_values($swap),$title);

return $title;

} // end getting other titles

function paged_comments_dupefix($content) { // remove post content if we're on a paged comment page
if ($this->get_comment_page() && $this->opt('paged_comments_dupefix')) {
	global $post;
	$content = '<p class="commentsactive">' . $this->opt_clean('comment_page_replacement') . '</p>';
	($this->opt('comment_page_replacement_override') && $this->opt('enable_secondary_titles')) ?
			   $title_for_insertion = $this->get_secondary_title() :
			   $title_for_insertion = single_post_title('',false);
	$post_link = '&#8220;<a href="' . get_permalink() . '">' . $title_for_insertion . '</a>&#8221;';
	$swaps = array (
				   "%post_title_linked%" => $post_link,
				   "%post_title%" => $title_for_insertion,
				   "%post_permalink%" => get_permalink(),
				   );
	$content = str_replace(array_keys($swaps), array_values($swaps), $content);
	return $this->prepout($content);
	}
else {return $content;}
} // end paged comments dupefix

function get_author() { // simple author grabber
global $wp_query;
$curauth = $wp_query->get_queried_object();
return $curauth->display_name;
} // end get author

function trimmer($totrim='',$length=160,$ellipsis='...') { // trim strings down to size
if (strlen($totrim) > $length) {
	$totrim = substr($totrim, 0, $length);
	$lastdot = strrpos($totrim, ".");
	$lastspace = strrpos($totrim, " ");
	$shorter = substr($totrim, 0, ($lastdot > $lastspace? $lastdot : $lastspace)); // truncate at either last dot or last space
	$shorter = rtrim($shorter, ' .') . $ellipsis; // trim off ending periods or spaces and append ellipsis
	} // end of snipping when too long
else { $shorter = $totrim; }
return $shorter;
} // end trimmer

function head_desc_comments() { // construct the head description for paged comment pages
global $post;
$desc = $this->opt_clean('paged_comments_meta_replacement');
$desc = str_replace('%post_title%',single_post_title('',false),$desc);
$desc = str_replace('%comment_page%',get_query_var('cpage'),$desc);
return $desc; // note value hasn't been texurized or escaped
} // end head desc for paged comments

function clean_fancies($content) { // get rid of some (space-wasting) common typographical fanciness
$replace = array (
		  "&ldquo;" => '"',
		  "&rdquo;"=> '"',
		  "&quot;" => '"',
		  "&lsquo;" => "'",
		  "&rsquo;" => "'",
		  "&mdash;" => "--",
		   );
return str_replace(array_keys($replace), array_values($replace), $content);
} // end cleaning out fancies

function head_desc() { // construct the head description
global $post,$paged;
if (is_404()) return;
if ($this->opt('paged_comments_meta_enable') && $this->get_comment_page()) {
	$description = $this->head_desc_comments();
	$custom = true;
	} // end handling comments pages
else
	{ // all the rest of this occurs only if we don't need a custom comments page meta
	$default  = get_bloginfo('name') . ': ' . get_bloginfo('description');
	$custom = $secondary_fallback = false;
	if (is_single() || is_page()) {
	   if ($this->opt('enable_alt_description')) {
		   $description = $this->get_meta_clean($post->ID,'alternative_description', true);
		   if ($description != '') $custom = true;
		   elseif ($this->opt('use_secondary_for_head')) $description = strip_tags($this->get_meta_clean($post->ID,'secondary_desc', true));
		   if ($description != '') $custom = $secondary_fallback = true; // flag will tell us if this was secondary description
		   elseif ($this->opt('enable_descriptions_legacy')) {
				  $description = get_post_meta($post->ID, 'description', true);
				  if ($description != '') $custom = true;
				  else {
					   $description = get_post_meta($post->ID, '_headspace_description', true);
					   if ($description != '') $custom = true;
					   } // end handling headspace
				  } // end handling legacy descriptions
		   } // end check for alt desc enabled
	   if (!$custom) { // no custom description?
		   $description_longer = $post->post_excerpt;
		   if ($description_longer == '') $description_longer = $post->post_content;
		   $description = trim(strip_tags(stripcslashes(str_replace(array("\r\n", "\r", "\n"), " ", $description_longer))));
		   } // end handling single or page but not custom
	   } // end handling single or page
	else
	   { // if not single or page...
	   $swap = array(
					"%blog_name%" => get_bloginfo('name'),
					"%blog_desc%" => get_bloginfo('description'),
					);
	   $key = $this->get_type_key();
	   $metaswaps = $this->get_swaps($key);
	   $suffix = '_meta_desc';   
	   if ($key != '') {
		   $description = $this->opt_clean($metaswaps[$key]['0'] . $suffix);
		   // Special handling for tag archives, so we can use a tag description if one is specified under 2.8+, or fall back to a different description if not
		   if (($key == 'tag') && ($metaswaps[$key]['1']['%tag_desc%'] != '') && $this->opt('tag_meta_desc_override'))
				$description = $this->opt_clean($metaswaps[$key]['0'] . $suffix . '_extra');
		   // end special handling for tag archives
		   $custom = true;
		   if (is_array($metaswaps[$key]['1'])) $swap = array_merge($swap,$metaswaps[$key]['1']);
		   }	
		   else $description = $default; // if it was none of these, just get name, desc
	   if (is_paged()) { // modify with something like a page number, if this is paged?
		   $modifier = $this->opt_clean($metaswaps['paged']['0'] . $suffix); // do some trickery to modify the title for paging
		   if ($modifier != '') $description = str_replace('%prior_meta_desc%',$description,$modifier); 
		   $swap = array_merge($swap,$metaswaps['paged']['1']);
		   } // end handling paged
	   $description = str_replace(array_keys($swap), array_values($swap),$description);
	   $description = wp_specialchars_decode($description, ENT_QUOTES); // just to make sure we don't send it out encoded twice
	   $description = strip_tags($this->strip_para($description)); // kill leftover markup
	   if ($description == '') $description = $default;
	   } // end handling other than single or page, now do stuff common to both
	
	} // end of handling other than comments pages

$description = $this->clean_fancies($description); // get rid of common typographical fanciness
$description = str_replace('"',"'",$description); // double quotes have to be htmlspecialchar-ed, but that wastes space in meta description
$description = preg_replace('/  +/',' ',$description); // get rid of extraneous spaces

$length = ('0' == $this->opt('desc_length')) ? 160 : $this->opt('desc_length');

if (((!$this->opt('desc_length_override')) && $custom) || !$custom || $secondary_fallback)
	 $description = $this->trimmer($description,$length); // only trim if not custom, or custom but not overriding

$description = htmlspecialchars($description);
$output = "<meta name=\"description\" content=\"{$description}\" />\n";
if ($this->opt('obnoxious_mode')) return $output;
else echo $output;
return;
} // end getting meta description

function legacy_keyword_cleanup ($list) { // clean up legacy lists of keywords
$list = stripslashes(wp_specialchars_decode($list,ENT_QUOTES));
// commented out following line because some plugins store without commas, some with; we will err on the side of too few commas rather than inserting them where they shouldn't be
// $list = str_replace(' ', ', ', $list);
$list = str_replace('"', '', $list);
$list = str_replace('+', ' ', $list);
$list = str_replace('_', ' ', $list);
$list = str_replace('-', ' ', $list);
$list = str_replace(',,', ',', $list);
return $list;
} // end legacy keyword cleanup

function head_keywords() { // construct head keyword list
global $wp_query,$post;
if (is_404()) return;
if ((!$this->opt('enable_keywords'))) return;
$defaults = $this->opt_clean('default_keywords');
if ($defaults == '')
	$defaults = get_bloginfo('name');
// if (PHP_VERSION >= 5)
// 	$temp_query = clone $wp_query; // note clone method is PHP5 only
// else $temp_query = $wp_query;
$taglist = '';
if (is_single() || is_page() ) : if ( have_posts() ) : while ( have_posts() ) : the_post(); // annoyingly, run a loop here to get what we need
if ($this->opt('enable_keywords_custom'))
	$taglist = $this->get_meta_clean($post->ID,'keywords', true);
if ($this->opt('enable_keywords_tags'))
	$posttags = get_the_tags();
if ($posttags) {
	if ($taglist != '') $taglist .= ', ';
	$showtags = array_slice($posttags,0,$this->opt('keyword_tags_limit')); // just keep the first specified number of tags
	foreach ($showtags as $tag) {
	   $taglist .= wp_specialchars_decode($tag->name,ENT_QUOTES) . ', ';
	   }
	$taglist = rtrim($taglist,', ');
} // end check for whether we have tags

if ($this->opt('enable_keywords_legacy')) {
// add in any custom field keywords
$supported = array('keyword','keywords','autometa','_headspace_keywords','_headspace_metakey');
foreach ($supported as $fieldname) {
	$extras = get_post_meta($post->ID, $fieldname, true);
	if ($extras != '') $taglist .= ', ' . $this->legacy_keyword_cleanup($extras);
	} // end loop for custom field keywords
} // end check for supporting legacy keywords

if ($taglist == '') $taglist = $defaults; // if nothing else, use defaults
if ($this->opt('enable_keywords_title')) $taglist = wp_specialchars_decode(strip_tags($post->post_title)) . ', ' . $taglist;

endwhile; endif; elseif(is_archive()):
	$taglist = $defaults;
	endif;
if (is_home()) {
	$homewords = $this->opt_clean('custom_home_keywords');
	$taglist = ('' == $homewords) ? $defaults : $homewords . ', ' . get_bloginfo('name');
} // end handling home
elseif(is_author())
	$taglist = $this->get_author() . ', ' . $taglist;
elseif(is_tag())
	$taglist = single_tag_title('',false) . ', ' . $taglist;
elseif(is_category())
	$taglist = stripslashes(wp_specialchars_decode(single_cat_title('',false),ENT_QUOTES)) . ', ' . $taglist;
if (trim($taglist,', ') == '') $taglist = $defaults;

// if (PHP_VERSION >= 5)
// 	$wp_query = clone $temp_query; // restore the original query so further loops are not messed up
// else $wp_query = $temp_query;

wp_reset_query(); // whew, thank goodness that's over

$taglist = htmlspecialchars(trim($this->trimmer($taglist,$this->opt('tags_length'),''), ', '));
$output = "<meta name=\"keywords\" content=\"{$taglist}\" />\n";
if ($this->opt('obnoxious_mode')) return $output;
else echo $output;
return;
} // end head keywords

function robots() { // construct head robots
global $post;
if (is_404()) return;
$tocheck = array ('author','category','search','tag','date');
$exclude = false;
foreach ($tocheck as $check) { // have we been told to exclude certain types of archives?
   $fx = 'is_' . $check;
   if ($fx() && $this->opt('index_' . $check . '_exclude')) $exclude = true;
} // end loop over types to check
if ($exclude) {
	$index = 'noindex';
	if ($this->opt('index_nofollow')) $index .= ',nofollow';
	} // end case for excluding
else $index = 'index,follow';
if ($this->opt('index_noodp')) $index .= ',noodp,noydir';
$output = "<meta name=\"robots\" content=\"{$index}\" />\n";
if ($this->opt('obnoxious_mode')) return $output;
else echo $output;
return;
} // end robots

} // end class definition

// Here's where we start executing

if (is_admin()) { // only load the admin stuff if we have to
   include ('ghpseo-setup-functions.php');
   include('ghpseo-writing.php');
   function ghpseo_setup_setngo() { // set up and instantiate admin class
	  $prefix = 'ghpseo';
	  $location_full = __FILE__;
	  $location_local = plugin_basename(__FILE__);
	  $options_page_details = array ('Greg&#8217;s HP SEO Options','High Performance SEO','gregs-high-performance-seo/ghpseo-options.php');
	  new ghpseoSetupHandler($prefix,$location_full,$location_local,$options_page_details);
	  } // end setup function
   ghpseo_setup_setngo();
   } // end admin-only stuff
else
   { // code for regular page views: instantiate class and provide interface function
   $ghpseo = new gregsHighPerformanceSEO('ghpseo');
   function ghpseo_output($type='main',$echo=true) {
	  global $ghpseo;
	  switch ($type) {
			  case "main": return $ghpseo->select_title(true,$echo);
			  case "main_title": return $ghpseo->select_title(true,$echo);
			  case "secondary_title": return $ghpseo->select_title(false,$echo);
			  case "description": return $ghpseo->select_desc($echo);
			  }
	  return;
	  }
   // last and least, if we're running with output buffering, set it up now:
   if ($ghpseo->opt('obnoxious_mode')) include ('ghpseo-sledgehammer-mode.php');
   } // end non-admin stuff

?>